@@ -1,13 +1,27 @@
 from plain.runtime import settings
-from plain.urls import path, reverse
+from plain.urls import Router, path, reverse
 
 from .fingerprints import get_fingerprinted_url_path
 from .views import AssetView
 
-default_namespace = "assets"
+
+class AssetsRouter(Router):
+    """
+    The router for serving static assets.
+
+    Include this router in your app router if you are serving assets yourself.
+    """
+
+    namespace = "assets"
+    urls = [
+        path("<path:path>", AssetView, name="asset"),
+    ]
 
 
-def get_asset_url(url_path):
+def get_asset_url(url_path: str) -> str:
+    """
+    Get the full URL to a given asset path.
+    """
     if settings.DEBUG:
         # In debug, we only ever use the original URL path.
         resolved_url_path = url_path
@@ -23,9 +37,4 @@ def get_asset_url(url_path):
     if settings.ASSETS_BASE_URL:
         return settings.ASSETS_BASE_URL + resolved_url_path
 
-    return reverse(default_namespace + ":asset", kwargs={"path": resolved_url_path})
-
-
-urlpatterns = [
-    path("<path:path>", AssetView, name="asset"),
-]
+    return reverse(AssetsRouter.namespace + ":asset", path=resolved_url_path)
