@@ -7,7 +7,7 @@ from .request import request
 
 @click.group("agent", invoke_without_command=True)
 @click.pass_context
-def agent(ctx):
+def agent(ctx: click.Context) -> None:
     """Tools for coding agents."""
     if ctx.invoked_subcommand is None:
         # If no subcommand provided, show all AGENTS.md files
