@@ -1,4 +1,11 @@
-def internalcode(obj):
+from __future__ import annotations
+
+from typing import TypeVar
+
+T = TypeVar("T")
+
+
+def internalcode(obj: T) -> T:
     """
     A decorator that simply marks a class or function as internal.
 
